"""Fetch all venues available."""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/venue.ipynb.

# %% auto 0
__all__ = ['fetch_all_venues']

# %% ../nbs/venue.ipynb 4
import pandas as pd

# %% ../nbs/venue.ipynb 5
def fetch_all_venues():
    """Fetches all venue information that exists: \n
    * venue_id \n
    * venue_name
    """

    try:
        print("Fetching all venues...")
        date_url = "https://raw.githubusercontent.com/seanyboi/rugbydata/main/data/v3/venue/venue_registry.parquet"
        venues = pd.read_parquet(date_url, engine="pyarrow")
        return venues
    except Exception as _:
        print(
            "No match information either because no venue took place or rugbypy does not have access to the venue data. Please raise an issue if neither."
        )



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/rugbypy',
                'doc_host': 'https://seanyboi.github.io',
                'git_url': 'https://github.com/seanyboi/rugbypy',
                'lib_path': 'rugbypy'},
  'syms': { 'rugbypy.competition': { 'rugbypy.competition.fetch_all_competitions': ( 'competition.html#fetch_all_competitions',
                                                                                     'rugbypy/competition.py')},
            'rugbypy.match': { 'rugbypy.match.fetch_all_matches': ('match.html#fetch_all_matches', 'rugbypy/match.py'),
                               'rugbypy.match.fetch_match_details': ('match.html#fetch_match_details', 'rugbypy/match.py'),
                               'rugbypy.match.fetch_matches': ('match.html#fetch_matches', 'rugbypy/match.py'),
                               'rugbypy.match.fetch_matches_by_date': ('match.html#fetch_matches_by_date', 'rugbypy/match.py')},
            'rugbypy.player': { 'rugbypy.player.cosine_similarity': ('player.html#cosine_similarity', 'rugbypy/player.py'),
                                'rugbypy.player.fetch_all_players': ('player.html#fetch_all_players', 'rugbypy/player.py'),
                                'rugbypy.player.fetch_player_id': ('player.html#fetch_player_id', 'rugbypy/player.py'),
                                'rugbypy.player.fetch_player_stats': ('player.html#fetch_player_stats', 'rugbypy/player.py'),
                                'rugbypy.player.word2vec': ('player.html#word2vec', 'rugbypy/player.py')},
            'rugbypy.team': { 'rugbypy.team.fetch_all_teams': ('team.html#fetch_all_teams', 'rugbypy/team.py'),
                              'rugbypy.team.fetch_team_stats': ('team.html#fetch_team_stats', 'rugbypy/team.py')},
            'rugbypy.venue': {'rugbypy.venue.fetch_all_venues': ('venue.html#fetch_all_venues', 'rugbypy/venue.py')}}}



"""Fetch team stats for individual games."""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/team.ipynb.

# %% auto 0
__all__ = ['fetch_all_teams', 'fetch_team_stats']

# %% ../nbs/team.ipynb 4
import pandas as pd
import typing

# %% ../nbs/team.ipynb 5
def fetch_all_teams():
    """
    Fetches all team available
    """
    try:
        print("Fetching all teams available...")
        team_url = "https://github.com/seanyboi/rugbydata/blob/main/data/v3/team/team_registry.parquet?raw=true"
        team_stats = pd.read_parquet(team_url, engine="pyarrow")
        return team_stats
    except Exception as _:
        print("No teams available. Please raise an issue!")

# %% ../nbs/team.ipynb 7
def fetch_team_stats(team_id: str, date: typing.Optional[str] = None):
    """
    Fetches all team stats for a particular team or if a date is passed (YYYYMMDD) then just for a particular game
    """
    try:
        if date:
            print(f"Fetching team stats for team_id: {team_id} on date: {date}...")
            team_url = f"https://github.com/seanyboi/rugbydata/blob/main/data/v3/team/{team_id}.parquet?raw=true"
            team_stats = pd.read_parquet(team_url, engine="pyarrow")
            team_stats = team_stats.query("game_date == @date")
            return team_stats
        else:
            print(f"Fetching all team stats for team_id: {team_id}...")
            team_url = f"https://github.com/seanyboi/rugbydata/blob/main/data/v3/team/{team_id}.parquet?raw=true"
            team_stats = pd.read_parquet(team_url, engine="pyarrow")
            return team_stats
    except Exception as _:
        print(
            "No team stats for {team_id} because the team id does not exist. Please raise an issue!"
        )



"""Fetch all competitions available."""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/competition.ipynb.

# %% auto 0
__all__ = ['fetch_all_competitions']

# %% ../nbs/competition.ipynb 4
import pandas as pd

# %% ../nbs/competition.ipynb 5
def fetch_all_competitions():
    """Fetches all competition information that exists: \n
    * match_id \n
    * home_team_id \n
    * away_team_id \n
    * date
    """

    try:
        print("Fetching all competition...")
        date_url = "https://raw.githubusercontent.com/seanyboi/rugbydata/main/data/v3/competition/competition_registry.parquet"
        competitions = pd.read_parquet(date_url, engine="pyarrow")
        return competitions
    except Exception as _:
        print(
            "No match information either because no competition took place or rugbypy does not have access to the competition data. Please raise an issue if neither."
        )



"""Fetch player starts for individual games."""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/player.ipynb.

# %% auto 0
__all__ = ['fetch_all_players', 'fetch_player_id', 'fetch_player_stats']

# %% ../nbs/player.ipynb 4
import pandas as pd
import typing
from collections import Counter
from math import sqrt

# %% ../nbs/player.ipynb 5
def fetch_all_players():
    """Fetches all players in the rugbypy manifest file."""
    try:
        player_manifest_url = "https://raw.githubusercontent.com/seanyboi/rugbydata/main/data/v3/player/player_registry.parquet"
        players = pd.read_parquet(player_manifest_url, engine="pyarrow")
        return players
    except Exception as _:
        print("No player manifest exists. Please raise an issue!")

# %% ../nbs/player.ipynb 7
def word2vec(word):

    # Count the number of characters in each word.
    count_characters = Counter(word)

    # Gets the set of characters and calculates the "length" of the vector.
    set_characters = set(count_characters)

    length = sqrt(sum(c * c for c in count_characters.values()))

    return count_characters, set_characters, length, word

# %% ../nbs/player.ipynb 8
def cosine_similarity(vector1, vector2, ndigits):

    # Get the common characters between the two character sets
    common_characters = vector1[1].intersection(vector2[1])

    # Sum of the product of each intersection character.
    product_summation = sum(
        vector1[0][character] * vector2[0][character] for character in common_characters
    )

    # Gets the length of each vector from the word2vec output.
    length = vector1[2] * vector2[2]

    # Calculates cosine similarity and rounds the value to ndigits decimal places.
    if length == 0:
        # Set value to 0 if word is empty.
        similarity = 0
    else:
        similarity = round(product_summation / length, ndigits)

    return similarity

# %% ../nbs/player.ipynb 9
def fetch_player_id(
    name: str,  # the name of the rugby player you wish to fetch the id for.
):
    """Uses a similarity tool to fetch a players id"""
    players = fetch_all_players()
    all_players = players["player_name"].to_list()
    # Initiate an empty list to store results.
    results_list = []

    # Apply word2vec function to each name and store them in a list.
    vector_list = [word2vec(str(i)) for i in all_players]
    vector_name = word2vec(str(name))
    # Two loops to compare each vector with another vector only once.
    for i in range(len(vector_list)):
        # Get first vector
        vector1 = vector_list[i]

        # Calculate cosine similarity
        similarity_score = cosine_similarity(vector1, vector_name, 3)

        # Append to results list if similarity score is between 1 and the threshold.
        # Note that scores of 1 can be ignored here if we want to exclude people with the same name.
        if 1 >= similarity_score >= 0.75:
            results_list.append([vector1[3], vector_name[3], similarity_score])

        else:
            pass
    # Convert list to dataframe.
    results_df = pd.DataFrame(results_list)
    if len(results_df) != 0:
        results_df.columns = ["player_name", "comparison_name", "similarity_score"]
        results_df = results_df.sort_values(
            by="similarity_score", ascending=False
        ).head(3)
        player_id = players.loc[
            players["player_name"].isin(results_df["player_name"].to_list())
        ]
        return player_id
    else:
        # Can add error here if there's no results to return if desired.
        print(
            f"Apologies, we could not find a match for {name}. Please try again or raise an issue!"
        )

# %% ../nbs/player.ipynb 11
def fetch_player_stats(
    player_id: str,  # the player id of a particular player.
    date: typing.Optional[
        str
    ] = None,  # the date of a particular match you wish to fetch the player stats for.
):
    """
    Fetches all player stats for a particular player or if a date is passed (YYYYMMDD) then just for a particular game.
    """
    try:
        if date:
            print(
                f"Fetching player stats for player_id: {player_id} on date: {date}..."
            )
            player_url = f"https://raw.githubusercontent.com/seanyboi/rugbydata/main/data/v3/player/{player_id}.parquet"
            player_stats = pd.read_parquet(player_url, engine="pyarrow")
            player_stats = player_stats.query("game_date == @date")
            return player_stats
        else:
            print(f"Fetching all player stats for player_id: {player_id}...")
            player_url = f"https://raw.githubusercontent.com/seanyboi/rugbydata/main/data/v3/player/{player_id}.parquet"
            player_stats = pd.read_parquet(player_url, engine="pyarrow")
            return player_stats
    except Exception as _:
        print(
            f"No player stats for {player_id} because the player id does not exist. Please raise an issue!"
        )



"""Fetch match stats for individual games."""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/match.ipynb.

# %% auto 0
__all__ = ['fetch_all_matches', 'fetch_matches_by_date', 'fetch_match_details', 'fetch_matches']

# %% ../nbs/match.ipynb 4
import pandas as pd

# %% ../nbs/match.ipynb 5
def fetch_all_matches():
    """Fetches all match information that exists: \n
    * match_id \n
    * home_team \n
    * away_team \n
    * date
    """

    try:
        print("Fetching all matches...")
        date_url = "https://raw.githubusercontent.com/seanyboi/rugbydata/main/data/v3/match/match_registry.parquet"
        matches = pd.read_parquet(date_url, engine="pyarrow").sort_values(
            by="date", ascending=True
        )
        return matches
    except Exception as e:
        print(
            f"No match information either because no matches took place or rugbypy does not have access to the match data - {e}. Please raise an issue if neither."
        )

# %% ../nbs/match.ipynb 7
def fetch_matches_by_date(date: str):
    """Fetches all match information on a particular date in the format YYYYMMDD that includes: \n
    * match_id \n
    * competition_id \n
    * home_team_id \n
    * home_team_name \n
    * away_team_id \n
    * away_team_name \n
    """

    try:
        print(f"Fetching matches on date: {date}...")
        date_url = f"https://raw.githubusercontent.com/seanyboi/rugbydata/main/data/v3/dates/{date}.parquet"
        matches = pd.read_parquet(date_url, engine="pyarrow")
        return matches
    except Exception as _:
        print(
            f"No match information for matches played on {date} either because no matches took place or rugbypy does not have access to the match data. Please raise en issue if neither."
        )

# %% ../nbs/match.ipynb 9
def fetch_match_details(match_id: str):
    """Fetches match data for a certain match_id"""
    print(f"Fetching match details for match_id: {match_id}...")
    try:
        match_url = f"https://raw.githubusercontent.com/seanyboi/rugbydata/main/data/v3/match/{match_id}.parquet"
        matches = pd.read_parquet(match_url, engine="pyarrow")
        return matches
    except Exception as _:
        print("Error fetching match data. Please raise an issue!")

# %% ../nbs/match.ipynb 11
def fetch_matches(date: str):
    print("Deprecated, please use the fetch_matches_by_date() function instead")
